<?php

namespace Drupal\performance_profiler\EventSubscriber;

use Drupal\Component\Utility\Timer;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;

/**
 * Subscribes to the kernel request event to register a shutdown function.
 */
class PerformanceProfilerEventSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Set a low value to start as early as possible.
    $events[KernelEvents::REQUEST][] = ['onRequest', -100];

    return $events;
  }

  /**
   * Register performance_profiler_shutdown function.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event response.
   */
  public function onRequest(GetResponseEvent $event) {
    Timer::start('performance_profiler');
    drupal_register_shutdown_function('performance_profiler_shutdown');
  }

}
