<?php

namespace Drupal\performance_profiler\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\performance_profiler\PerformanceBenchmark;

/**
 * Configure Memory profiler module settings.
 *
 * @internal
 */
class PerformanceProfilerForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'performance_profiler_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['performance_profiler.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $site_config = $this->config('performance_profiler.settings');
    $moduleHandler = \Drupal::service('module_handler');

    $form['appearence'] = [
      '#type' => 'details',
      '#title' => $this->t('Appearance'),
      '#open' => TRUE,
    ];
    $form['appearence']['watchdog'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log statistics into watchdog'),
      '#default_value' => $site_config->get('watchdog'),
      '#description' => $this->t('If checked, the message will be logged via watchdog system.'),
    ];
    // @todo check if toolbar module enabled
    $form['appearence']['toolbar'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log statistics into Toolbar'),
      '#default_value' => $site_config->get('toolbar'),
      '#description' => $this->t('If checked, the message will be printed at the Toolbar header.'),
    ];
    if (!$moduleHandler->moduleExists('toolbar')) {
      $form['appearence']['toolbar']['#attributes'] = ['disabled' => 'disabled'];
      $form['appearence']['toolbar']['#default_value'] = 0;
    }
    $form['appearence']['popup'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log statistics into Popup'),
      '#default_value' => $site_config->get('popup'),
      '#description' => $this->t('If checked, the message will be printed at the bottom right conner.'),
    ];

    $form['appearence']['anonymous'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log statistics for anonymous user'),
      '#default_value' => $site_config->get('anonymous'),
      '#description' => $this->t('If checked, each request from anonymous user will be logged.'),
    ];

    $form['entries'] = [
      '#type' => 'details',
      '#title' => $this->t('Log entries'),
      '#open' => TRUE,
    ];
    $form['entries']['self'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log self ajax query'),
      '#default_value' => $site_config->get('self'),
      '#description' => $this->t('Log query to get data from Performance profiler, usually should be disabled.'),
    ];
    $form['entries']['ajax'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log AJAX queries'),
      '#default_value' => $site_config->get('ajax'),
      '#description' => $this->t('Log XHR queries (contextual, overlay, etc).'),
    ];

    $form['track'] = [
      '#type' => 'details',
      '#title' => $this->t('Track'),
      '#open' => TRUE,
    ];
    $form['track']['memory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Min memory usage to track'),
      '#default_value' => $site_config->get('memory'),
      '#description' => $this->t('If not empty or more then 0, will be tracked only higher values (in megabytes).'),
    ];
    $form['track']['time'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Min execution time to track'),
      '#default_value' => $site_config->get('time'),
      '#description' => $this->t('If not empty or more then 0, will be tracked only higher values (e.g. 4 or 5.5).'),
    ];

    $form['actions']['bench_php'] = [
      '#type' => 'submit',
      '#value' => $this->t('Run PHP benchmark'),
      '#button_type' => 'secondary',
      '#submit' => ['::benchmarkPhp'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('performance_profiler.settings')
      ->set('watchdog', $form_state->getValue('watchdog'))
      ->set('toolbar', $form_state->getValue('toolbar'))
      ->set('popup', $form_state->getValue('popup'))
      ->set('anonymous', $form_state->getValue('anonymous'))
      ->set('self', $form_state->getValue('self'))
      ->set('ajax', $form_state->getValue('ajax'))
      ->set('memory', $form_state->getValue('memory'))
      ->set('time', $form_state->getValue('time'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function benchmarkPhp(array &$form, FormStateInterface $form_state) {
    foreach (PerformanceBenchmark::run(FALSE) as $value) {
      \Drupal::messenger()->addMessage($value);
    }
  }

}
