<?php

namespace Drupal\performance_profiler;

/*
//
// PHP Benchmark Performance Script                  #
// � 2010 Code24 BV                               #
//
// Author      : Alessandro Torrisi                                      #
// Company     : Code24 BV, The Netherlands                              #
// Date        : July 31, 2010                                           #
// version     : 1.0.1                                                   #
// License     : Creative Commons CC-BY license                          #
// Website     : http://www.php-benchmark-script.com                     #
//
// .
 */

/**
 * Provides a PerformanceBenchmark Class to run tests.
 */
class PerformanceBenchmark {

  /**
   * Provides test scripts.
   */
  private static function testMath($count = 140000) {
    $time_start = microtime(TRUE);
    $mathFunctions = [
      "abs",
      "acos",
      "asin",
      "atan",
      "floor",
      "exp",
      "sin",
      "tan",
      "is_finite",
      "is_nan",
      "sqrt",
    ];
    foreach ($mathFunctions as $key => $function) {
      if (!function_exists($function)) {
        unset($mathFunctions[$key]);
      }
    }
    for ($i = 0; $i < $count; $i++) {
      foreach ($mathFunctions as $function) {
        switch ($function) {
          case 'bindec':
            call_user_func($function, $i % 2);
            break;

          case 'pi':
            call_user_func($function);
            break;

          default:
           call_user_func_array($function, [$i]);
        }
      }
    }
    return number_format(microtime(TRUE) - $time_start, 3);
  }

  /**
   * Performs string manipulation tests.
   */
  private static function testStringManipulation($count = 130000) {
    $time_start = microtime(TRUE);
    $stringFunctions = [
      "addslashes",
      "chunk_split",
      "metaphone",
      "strip_tags",
      "md5",
      "sha1",
      "strtoupper",
      "strtolower",
      "strrev",
      "strlen",
      "soundex",
      "ord",
    ];
    foreach ($stringFunctions as $key => $function) {
      if (!function_exists($function)) {
        unset($stringFunctions[$key]);
      }
    }
    $string = "the quick brown fox jumps over the lazy dog";
    for ($i = 0; $i < $count; $i++) {
      foreach ($stringFunctions as $function) {
        call_user_func_array($function, [$string]);
      }
    }
    return number_format(microtime(TRUE) - $time_start, 3);
  }

  /**
   * Provides test loops.
   */
  private static function testLoops($count = 19000000) {
    $time_start = microtime(TRUE);
    for ($i = 0; $i < $count; ++$i) {
      ;
    }
    $i = 0;
    while ($i < $count) {
      ++$i;
    }
    return number_format(microtime(TRUE) - $time_start, 3);
  }

  /**
   * Provides if/else tests.
   */
  private static function testIfElse($count = 9000000) {
    $time_start = microtime(TRUE);
    for ($i = 0; $i < $count; $i++) {
      if ($i == -1) {
      }
      elseif ($i == -2) {
      }
      else {
        if ($i == -3) {
        }
      }
    }
    return number_format(microtime(TRUE) - $time_start, 3);
  }

  /**
   * Runs the tests.
   */
  public static function run($echo = TRUE) {
    $total = 0;
    $server = (isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : '?') . '@' . (isset($_SERVER['SERVER_ADDR']) ? $_SERVER['SERVER_ADDR'] : '?');
    $methods = get_class_methods(self::class);
    $line = str_pad("-", 38, "-");
    $return_echo = "$line\n|" . str_pad("PHP BENCHMARK SCRIPT", 36, " ", STR_PAD_BOTH) . "|\n$line\nStart : " . date("Y-m-d H:i:s") . "\nServer : $server\nPHP version : " . PHP_VERSION . "\nPlatform : " . PHP_OS . "\n$line\n";
    $return = [];
    foreach ($methods as $method) {
      if (preg_match('/^test/', $method)) {
        $total += $result = self::$method();
        $return_echo .= $return[] = str_pad($method, 25) . " : " . $result . " sec.\n";
      }
    }
    $return_echo .= str_pad("-", 38, "-") . "\n" . ($return[] = str_pad("Total time:", 25) . " : " . $total . " sec.\n");
    if ($echo) {
      echo $return_echo;
    }
    return $return;
  }

}
